@@ -34,6 +34,7 @@
 #include <DSysInfo>
 #include <QDBusInterface>
 #include <QRegExp>
+#include <QUuid>
 
 #include "base/consts.h"
 #include "service/settings_name.h"
@@ -47,7 +48,6 @@ namespace installer {
 namespace {
 
 // Absolute path to oem dir.
-QString g_oem_dir;
 
 const char kLocaleKey[] = "DI_LOCALE";
 const char kComponentUninstallPackages[] = "DI_COMPONENT_UNINSTALL";
@@ -113,20 +113,21 @@ QStringList ListAvatarFiles(const QString& dir_name) {
 }  // namespace
 
 QDir GetOemDir() {
-  if (g_oem_dir.isEmpty()) {
+    QString g_oem_dir;
     if (QDir(kDebugOemDir).exists()) {
-      g_oem_dir = kDebugOemDir;
+        g_oem_dir = kDebugOemDir;
     } else if (QDir(kUbuntuOemDir).exists()) {
-      g_oem_dir = kUbuntuOemDir;
+        g_oem_dir = kUbuntuOemDir;
     } else if (QDir(kDeepinOemDir).exists()) {
-      g_oem_dir = kDeepinOemDir;
+        g_oem_dir = kDeepinOemDir;
     } else if (QDir(kDeepinOemCandidateDir).exists()) {
-      g_oem_dir = kDeepinOemCandidateDir;
+        g_oem_dir = kDeepinOemCandidateDir;
     } else {
-      g_oem_dir = kDeepinOemCandidateDir2;
+        g_oem_dir = kDeepinOemCandidateDir2;
     }
-  }
-  return QDir(g_oem_dir);
+
+    qDebug() << "OEM_DIR=" << g_oem_dir;
+    return QDir(g_oem_dir);
 }
 
 QString GetOemLicenseDir() {
@@ -157,7 +158,7 @@ QString GetOSType()
 #ifdef QT_DEBUG
     QString type = "Professional";
 #else
-    QString type = settings.value("Type", "Community").toString();
+    QString type = settings.value("Type", "Desktop").toString();
 #endif // QT_DEBUG
 
     return type;
@@ -167,10 +168,11 @@ OSType GetCurrentType() {
     const QString& type = GetOSType();
 
     return QMap<QString, OSType>{
-        { "Community", OSType::Community },
+        { "Desktop", OSType::Community },
         { "Professional", OSType::Professional },
         { "Server", OSType::Server },
         { "Personal", OSType::Personal },
+        { "Device", OSType::Device }
     }[type];
 }
 
@@ -334,16 +336,6 @@ QString GetOemCheckHooksDir() {
     return GetOemDir().absoluteFilePath("check_hooks");
 }
 
-QString GetReservedUsernameFile() {
-  const QString oem_file = GetOemDir().absoluteFilePath("reserved_usernames");
-  if (QFile::exists(oem_file)) {
-    return oem_file;
-  }
-
-  // Returns default list.
-  return RESOURCES_DIR "/reserved_usernames";
-}
-
 QString GetVendorLogo() {
   QString oem_file = DSysInfo::distributionOrgLogo(DSysInfo::Distribution, DSysInfo::Normal);
   qDebug() << "oem_file = " << oem_file;
@@ -357,6 +349,12 @@ QString GetVendorLogo() {
     return oem_file;
   }
 
+  oem_file = GetOemDir().absoluteFilePath("vendor.svg");
+  qDebug() << "oem_file = " << oem_file;
+  if (QFile::exists(oem_file)) {
+    return oem_file;
+  }
+
   // Returns builtin logo.
   return ":/images/logo.svg";
 }
@@ -672,6 +670,8 @@ void AddConfigFile() {
         case OSType::Professional: prefix = "professional"; break;
         case OSType::Server: prefix = "server"; break;
         case OSType::Personal: prefix = "personal"; break;
+        case OSType::Device:prefix = "device";break;
+        default: prefix = "community";break;
     }
 
 #ifdef QT_DEBUG
@@ -710,6 +710,10 @@ void AddConfigFile() {
 
 void WriteFullDiskEncryptPassword(const QString &password)
 {
+    if (password.isEmpty()) {
+        qInfo() << "Encrypt password is empty";
+    }
+
     AppendToConfigFile("DI_CRYPT_PASSWD", password);
 }
 
@@ -717,8 +721,8 @@ void BeforeInstallHook()
 {
     const QString subPath = "/hooks/before_install/";
     QStringList jobList;
-    const QStringList pathList{kDeepinOemCandidateDir, kDeepinOemDir, kUbuntuOemDir, kDebugOemDir};
-    QStringList nameFilters;
+    const QString oemdirpath = installer::GetOemDir().path();
+    QStringList nameFilters {"*.job"};
 
     QString objPath = "/tmp/installer/";
     QDir objDir(objPath);
@@ -726,13 +730,14 @@ void BeforeInstallHook()
         objDir.mkpath(objPath);
     }
 
-    //all type file
-    nameFilters << "*";
+    QStringList pathlist = {"/usr/share/deepin-installer/hooks/before_install/", oemdirpath + subPath};
 
-    for (int i = 0; i < pathList.size(); i++) {
-        QString path = pathList.at(i) + subPath;
+    for (int i =0; i< pathlist.size(); i++) {
+        QString path = pathlist.at(i);
+        qDebug() << "before installer job path:" << path;
         QDir dir(path);
-        QStringList files = dir.entryList(nameFilters, QDir::Files|QDir::Executable, QDir::Name);
+        QStringList const files = dir.entryList(nameFilters, QDir::Files);
+        qDebug() << "before installer job list:" << files;
         foreach (const QString& f, files) {
             const QString fileName = path + f;
             QFile file(objPath + f);
@@ -742,16 +747,15 @@ void BeforeInstallHook()
             if (!QFile::copy(fileName, objPath + f)) {
                 qCritical() << "copy " << fileName << " to " << objPath << " failed!";
             }
-
         }
     }
 
     //TODO
     //Splicing according to the actual situation
     const QString hookManager = "/usr/share/deepin-installer/hooks/hook_manager.sh";
-    QDir dir(objPath);
-    QStringList files = dir.entryList(nameFilters, QDir::Files|QDir::Executable, QDir::Name);
-    foreach (const QString job, files) {
+    QDir objdir(objPath);
+    QStringList objfiles = objdir.entryList(nameFilters, QDir::Files);
+    foreach (const QString job, objfiles) {
         if (!SpawnCmd(hookManager, {objPath + job})) {
             qCritical() << "hook_manager.sh " << job << " failed";
             return;
@@ -825,42 +829,84 @@ void WriteIsEnableNTP(bool isEnableNTP)
 
 void WriteFullDiskResolution(const FinalFullDiskResolution& resolution)
 {
-    const char kFullDiskResolutionDeviceSeparator[] = ";";
-    QString key { "" };
     QStringList valueList;
     for (int i = 0; i < resolution.option_list.length(); i++) {
         valueList << resolution.option_list.at(i).device;
     }
-    AppendToConfigFile("DI_FULLDISK_MULTIDISK_DEVICE", valueList.join(kFullDiskResolutionDeviceSeparator));
 
-    QString labelKey { "" };
-    QStringList labelValueList;
+    if (valueList.isEmpty()) {
+        // 无人值守，重新设置一遍该值，防止在无人值守的时候没有配置DI_FULLDISK_MULTIDISK_DEVICE选项，导致系统安装报错
+        QString devices = GetSettingsString("DI_FULLDISK_MULTIDISK_DEVICE");
+        AppendToConfigFile("DI_FULLDISK_MULTIDISK_DEVICE", devices);
+
+    } else {
+        // 非无人值守，通过界面设置设备
+        AppendToConfigFile("DI_FULLDISK_MULTIDISK_DEVICE", valueList.join(";"));
+    }
+
     for (int i = 0; i < resolution.option_list.length(); i++) {
         const FinalFullDiskOption& op = resolution.option_list[i];
-        key = QString("DI_FULLDISK_MULTIDISK_POLICY_%1").arg(i);
-        labelKey = QString("DI_FULLDISK_MULTIDISK_LABEL_%1").arg(i);
-        valueList.clear();
-        labelValueList.clear();
+        QString key = QString("DI_FULLDISK_MULTIDISK_POLICY_%1").arg(i);
+        QString labelKey = QString("DI_FULLDISK_MULTIDISK_LABEL_%1").arg(i);
+        QString ext_key = QString("DI_FULLDISK_MULTIDISK_EXTSIZE_%1").arg(i);
+
+        // 占位符
+        QString placeholder = "%1";
+
+        int AvaLVMSize = 0;
+        int AvaExtSize = 0;
+
+        QString vg_info = QString("");
+        QString vg_labe_info = QString("");
+
+        QStringList partInfoList = {placeholder};
+        QStringList labelValueList = {placeholder};      
 
-        if (i > 0 && op.encrypt) {
-            valueList << QString("luks_crypt%1:crypto_luks::100%").arg(i);
-            labelValueList << QString("luks_crypt%1").arg(i);
-        }
         for (int j = 0; j < op.policy_list.length(); j++) {
             const FinalFullDiskPolicy& policy = op.policy_list[j];
-            valueList << QString("%1:%2:%3:%4")
+            QString info = QString("%1:%2:%3:%4")
                     .arg(policy.mountPoint)
                     .arg(policy.filesystem)
                     .arg(policy.offset/kMebiByte)
                     .arg(policy.size/kMebiByte);
-            labelValueList << policy.label;
-            if (op.encrypt && 0 == i && policy.mountPoint == "/boot") {
-                valueList << QString("luks_crypt%1:crypto_luks::100%").arg(i);
-                labelValueList << QString("luks_crypt%1").arg(i);
+
+            if (policy.mountPoint == "/boot" || policy.mountPoint == "/boot/efi") {
+                // 全盘加密信息左边插入 - 普通分区
+                partInfoList.insert(partInfoList.indexOf(placeholder), info);
+                labelValueList.insert(labelValueList.indexOf(placeholder), policy.label);
+
+            } else if (op.encrypt) {
+                // 全盘加密信息右边插入 - lvm分区
+                partInfoList.append(info);
+                labelValueList.append(policy.label);
+
+                vg_info = QString("luks_crypt%1:crypto_luks::100%").arg(i);
+                vg_labe_info = QString("luks_crypt%1").arg(i);
+
             }
+            else if (policy.isLvm) {
+                // lvm分区信息右边插入 - lvm分区
+                partInfoList.append(info);
+                labelValueList.append(policy.label);
+
+                AvaLVMSize += policy.size/kMebiByte;
+                // vg name需要唯一，防止多盘情况下两次安装出现名字重复导致分区失败
+                vg_info = QString("%1_%2:lvm_type::%3").arg(QUuid::createUuid().toString().remove("{").remove("}"), \
+                                                            QString::number(i), QString::number(AvaLVMSize));
+                vg_labe_info = QString("%1_%2").arg(QUuid::createUuid().toString(), QString::number(i));
+            }
+            else {
+                // 全盘加密信息左边插入 - 普通分区
+                AvaExtSize += policy.size/kMebiByte;
+                partInfoList.insert(partInfoList.indexOf(placeholder), info);
+                labelValueList.insert(labelValueList.indexOf(placeholder), policy.label);
+            }
+
         }
-        AppendToConfigFile(key, valueList.join(kFullDiskResolutionDeviceSeparator));
-        AppendToConfigFile(labelKey, labelValueList.join(kFullDiskResolutionDeviceSeparator));
+
+        AppendToConfigFile(ext_key, QString::number(AvaExtSize));
+        AppendToConfigFile(key, partInfoList.join(";").arg(vg_info));
+        AppendToConfigFile(labelKey, labelValueList.join(";").arg(vg_labe_info));
     }
     WriteFullDiskMode(true);
 }
@@ -1019,4 +1065,9 @@ void ScanNetDevice()
 
 }
 
+void WriteSaveUserData(bool issave)
+{
+    AppendToConfigFile("DI_SAVE_DATA", issave);
+}
+
 }  // namespace installer
