@@ -94,7 +94,7 @@ bool FullDiskDelegate::formatWholeDevice(const QString& device_path,
   qint64         lastDeviceLenght{ device->length };
   Partition::Ptr unallocated = device->partitions.last();
 
-  if (device->table == PartitionTableType::GPT) {
+  if (device->table == PartitionTableType::GPT && GetSettingsBool(kSystemIsCreateEFI)) {
       // 首先创建EFI分区
       const qint64 uefiSize =
           ParsePartitionSize("300Mib", lastDeviceLenght * device->sector_size);
@@ -304,10 +304,10 @@ bool FullDiskDelegate::formatWholeDeviceMultipleDisk()
     PartitionTableType table;
     table = IsEfiEnabled() ? PartitionTableType::GPT : PartitionTableType::MsDos;
 
-    FullDiskOption   disk_option;
+    FullDiskOption disk_option;
     FullDiskPolicyList & policy_list = disk_option.policy_list;
 
-    QStringList device_path_list;
+    QStringList device_path_list = selected_disks;
     // Get All policies from json file.
     {
         const QByteArray& policyStr{ GetFullDiskInstallPolicy() };
@@ -336,21 +336,14 @@ bool FullDiskDelegate::formatWholeDeviceMultipleDisk()
             policy.alignStart = jsonObject["alignStart"].toBool();
             policy.device     = jsonObject["device"].toString();
             policy.isDataPartition = jsonObject["isDataPartition"].toBool();
+            policy.isLvm      = jsonObject["isLvm"].toBool();
             if (policy.mountPoint == kLinuxSwapMountPoint) {
                 policy.mountPoint = "";
             }
             policy_list.push_back(policy);
-
-            if (!policy.device.isEmpty() && device_path_list.indexOf(policy.device) == -1) {
-                device_path_list.append(policy.device);
-            }
         }
     }
 
-    if (device_path_list.isEmpty()) {
-        device_path_list = selected_disks;
-    }
-
     if (device_path_list.isEmpty()) {
         qWarning() << Q_FUNC_INFO << "select Disk is Empty";
         return false;
@@ -381,15 +374,19 @@ bool FullDiskDelegate::formatWholeDeviceMultipleDisk()
         // 默认json的device字段为空
         if (device_path_list.length() > 1
             && policy.device.isEmpty()
-            && policy.isDataPartition) {
+            && (policy.isDataPartition || policy.label == "_dde_data")) {
 
             policy.device = device_path_list.at(1);
         }
         else if (policy.device.isEmpty()){
             policy.device = device_path_list.at(0);
         }
+
+        qDebug() << "Json config info:" << "Label:" << policy.label << "device:" << policy.device;
     }
 
+    m_FullDiskPolicyList = policy_list;
+
     selected_devices.clear();
     // Format every disks one by one.
     for (const QString& device_path : device_path_list) {
@@ -399,6 +396,7 @@ bool FullDiskDelegate::formatWholeDeviceMultipleDisk()
         device->partitions.clear();
         device->table = table;
 
+        qInfo() << "scan device info: " << device->path << device->length;
         disk_option.is_system_disk = device->path == device_path_list.at(0);
         if (!formatWholeDeviceV2(device, disk_option)) {
             selected_devices.clear();
@@ -412,7 +410,8 @@ bool FullDiskDelegate::formatWholeDeviceMultipleDisk()
 struct  FullDiskPolicyComparator {
     bool operator()(const FullDiskPolicy& a, const FullDiskPolicy& b)
     {
-            return a.startSector < b.startSector;
+        // (b.label != "SWAP" && b.label != "Backup") ||
+        return  (a.startSector < b.startSector);
     }
 };
 
@@ -423,6 +422,7 @@ bool FullDiskDelegate::formatWholeDeviceV2(const Device::Ptr& device, FullDiskOp
     operation->applyToVisual(device);
 
     int            primary_count { 0 };
+    bool           hasVG = false;
     const uint     swapSize{ getSwapSize() };
     qint64         startSector{ 0 };
     qint64         endSector { device->length - 1 };
@@ -430,13 +430,22 @@ bool FullDiskDelegate::formatWholeDeviceV2(const Device::Ptr& device, FullDiskOp
     Partition::Ptr unallocated = device->partitions.last();
 
     const qint64 oneMebiByteSector = 1 * kMebiByte / device->sector_size;
-    startSector = oneMebiByteSector;
-    lastDeviceLenght -= oneMebiByteSector;
+    if ( GetSettingsBool("multi_system_eanble") ){
+        qint64 startSectorfromshell = GetSettingsString("DI_INSTALL_STARTPOS").toLongLong();
+        startSector = startSectorfromshell / kKibiByte / 2 ;
+        device->partitions[0]->start_sector = startSector;
+        lastDeviceLenght -= startSectorfromshell;
+        qWarning() << "lastDeviceLenght is : " << lastDeviceLenght << "DI_INSTALL_STARTPOS is : " << startSectorfromshell << "startSector is : " << startSector << "device->sector_size is : " << device->sector_size;
+    } else {
+        startSector = oneMebiByteSector;
+        lastDeviceLenght -= oneMebiByteSector;
+    }
+
     qint64 adjust_start_offset_sector = startSector;
     int root_size_count { 0 };
     int percent100_count { 0 };
 
-    if (IsEfiEnabled() && option.is_system_disk && GetCurrentPlatform() != "sw") {
+    if (IsEfiEnabled() && option.is_system_disk && GetCurrentPlatform() != "sw" && GetSettingsBool(kSystemIsCreateEFI) && !GetSettingsBool("multi_system_eanble")) {
         const qint64 uefiSize =
             ParsePartitionSize("300Mib", lastDeviceLenght * device->sector_size);
         if (!createPrimaryPartition(unallocated, PartitionType::Normal, true, FsType::EFI,
@@ -484,36 +493,25 @@ bool FullDiskDelegate::formatWholeDeviceV2(const Device::Ptr& device, FullDiskOp
             partitionSize = ParsePartitionSize(swap_size_str,length);
         }
 
-        if (policy.usage == "backup-size") {
+        else if (policy.usage == "backup-size") {
             //恢复分区大小=总分区大小的2%+10GB
-            qint64  backSize = length * 2 / 100 / kGibiByte + 10;
+            qint64  backSize = length * 2 / 100 / kGibiByte + GetSettingsInt(kSystemRecoveryBaseSize);
             const QString backup_size_str = QString("%1gib").arg(backSize);
             partitionSize = ParsePartitionSize(backup_size_str,length);
-         }
 
-        if (policy.usage == "server-backup-size") {
-            //恢复分区大小=总分区大小的2%+20GB
-            qint64  backSize = length * 2 / 100 / kGibiByte + 20;
-            const QString backup_size_str = QString("%1gib").arg(backSize);
-            partitionSize = ParsePartitionSize(backup_size_str,length);
-        }
-
-        if (policy.usage == "100%") {
+        } else if (policy.usage == "100%") {
             percent100_count++;
-        }
 
-        if (policy.mountPoint == kMountPointRoot || policy.label == kFullDiskPolicyRootb) {
-            if (policy.usage == kFullDiskPolicyUsageRootSize) {
-                root_size_count++;
-                const QString root_size_str = GetSettingsString(kPartitionFullDiskRootPartitionUsage);
-                partitionSize = ParsePartitionSize(root_size_str, device->getByteLength());
-                qDebug() << "zdd test1: partitionSize" << partitionSize << " ||| "<< policy.mountPoint << policy.sectors << policy.label;
-            }
+        } else if (policy.usage == kFullDiskPolicyUsageRootSize) {
+            // root_siez 大小一般为磁盘的20%,并且在partition_full_disk_large_root_part_range配置的范围内
+            const QString root_size_str = GetSettingsString(kPartitionFullDiskRootPartitionUsage);
+            partitionSize = ParsePartitionSize(root_size_str, device->getByteLength());
+
 
             root_range = getRootPartitionSizeRange();
             partitionSize = std::max(partitionSize, root_range.min_size_bytes);
             partitionSize = std::min(partitionSize, root_range.max_size_bytes);
-            qDebug() << "zdd test2: partitionSize" << partitionSize << " ||| "<< policy.mountPoint << policy.sectors << policy.label;
+
         }
 
         if (partitionSize < 1) {
@@ -531,21 +529,37 @@ bool FullDiskDelegate::formatWholeDeviceV2(const Device::Ptr& device, FullDiskOp
             policy.startSector = policy.endSector - policy.sectors + 1;
             endSector -= policy.sectors;
         }
+        qDebug() << "device legth: " << lastDeviceLenght
+                 << ". policy.label=" << policy.label << ":" << policy.sectors;
 
         lastDeviceLenght -= policy.sectors;
         if (lastDeviceLenght < 0) {
-            qDebug() << QString("FULLDISK:size out of range. please check your configuration file");
+            qDebug() << QString("FULLDISK:size out of range. please check your configuration file. lastDeviceLenght= ")
+                     << lastDeviceLenght
+                     << ". policy.label=" << policy.label << ":" << policy.sectors;
             return false;
         }
 
         bool is_primary = ((device->table == PartitionTableType::GPT) ||
-                           (primary_count < (device->max_prims - 1)));
+                           (primary_count < (device->max_prims - 1))  ||
+                           policy.isLvm);
         policy.partitionType =
             is_primary ? PartitionType::Normal : PartitionType::Logical;
 
-        if (is_primary) {
+        if (policy.isLvm && !hasVG) {
+            hasVG = true;
             primary_count++;
         }
+
+        if (!policy.isLvm && is_primary) {
+            primary_count++;
+        }
+
+        if (policy.mountPoint == kMountPointRoot || policy.label == kFullDiskPolicyRootb) {
+            root_size_count++;
+        }
+
+        qInfo() << "FullDiskPolicy start sector old info: " << policy.label << ": " << policy.startSector;
     }
 
     // Adjust partition size for root partition if it is system disk
@@ -559,10 +573,11 @@ bool FullDiskDelegate::formatWholeDeviceV2(const Device::Ptr& device, FullDiskOp
     adjustFullDiskPolicy(device, option, adjust);
 
     // sort all policies
-    qSort(begin, end, FullDiskPolicyComparator());
+    qSort(policy_list.begin(), policy_list.end(), FullDiskPolicyComparator());
 
     const FullDiskPolicyList& const_policy_list = option.policy_list;
     for (const FullDiskPolicy& policy : const_policy_list) {
+        qInfo() << "FullDiskPolicy start sector new info: " << policy.label << ": " << policy.startSector;
         if (policy.device != device->path) {
             continue;
         }
@@ -571,7 +586,7 @@ bool FullDiskDelegate::formatWholeDeviceV2(const Device::Ptr& device, FullDiskOp
                  << policy.label;
         // alignStart is always true ,since policies are sorted by start_sector
         if (!createPartition(unallocated, policy.partitionType, true, policy.filesystem,
-                    policy.mountPoint, policy.sectors,policy.label)) {
+                    policy.mountPoint, policy.sectors,policy.label, policy.isLvm)) {
             return false;
         }
 
@@ -595,12 +610,13 @@ void FullDiskDelegate::removeAllSelectedDisks()
     selected_disks.clear();
 }
 
-void FullDiskDelegate::addSystemDisk(const QString & device_path)
+void FullDiskDelegate::addSystemDisk(const QString &device_path)
 {
     selected_devices.clear();
     selected_disks.clear();
     selected_disks.append(device_path);
 
+
     qInfo() << Q_FUNC_INFO << "add system disk: " << device_path;
 }
 
@@ -640,6 +656,7 @@ void FullDiskDelegate::getFinalDiskResolution(FinalFullDiskResolution& resolutio
         policy.size = (op->new_partition->end_sector - op->new_partition->start_sector) *
                 op->new_partition->sector_size;
         policy.device = op->new_partition->device_path;
+        policy.isLvm = op->new_partition->is_lvm;
 
         if (option.policy_list.length() > 0
             && option.policy_list.last().device != policy.device) {
@@ -662,6 +679,18 @@ void FullDiskDelegate::setAutoInstall(bool autoinstall)
     m_autoInstall = autoinstall;
 }
 
+bool FullDiskDelegate::isLvm()
+{
+    for (int i = 0; i < m_FullDiskPolicyList.size(); i++) {
+        qDebug() << "m_FullDiskPolicyList: " << m_FullDiskPolicyList.at(i).label << "  " << m_FullDiskPolicyList.at(i).isLvm;
+        if (m_FullDiskPolicyList.at(i).isLvm) {
+            return true;
+        }
+    }
+
+    return false;
+}
+
 void FullDiskDelegate::onDeviceRefreshed(const DeviceList &devices)
 {
     partition::Delegate::onDeviceRefreshed(devices);
