@@ -85,11 +85,6 @@ bool AdvancedPartitionFrame::validate() {
   }
 }
 
-bool AdvancedPartitionFrame::isInstallNvidia() const
-{
-    return m_installNvidiaCheck->isChecked();
-}
-
 QList<Device::Ptr> AdvancedPartitionFrame::getAllUsedDevice() const
 {
     return delegate_->getAllUsedDevice();
@@ -228,12 +223,6 @@ void AdvancedPartitionFrame::setBootloaderPath(const QString& bootloader_path) {
   bootloader_button_->setText(bootloader_path);
 }
 
-void AdvancedPartitionFrame::installNvidiaStateChanged(bool install_nvidia) {
-    if (!delegate_->m_islvm) {
-        WriteEnableNvidiaDriver(install_nvidia);
-    }
-}
-
 void AdvancedPartitionFrame::changeEvent(QEvent* event) {
   if (event->type() == QEvent::LanguageChange) {
     bootloader_tip_button_->setText(::QObject::tr("Change boot loader"));
@@ -242,7 +231,6 @@ void AdvancedPartitionFrame::changeEvent(QEvent* event) {
     } else {
       editing_button_->setText(::QObject::tr("Delete"));
     }
-    m_installNvidiaCheck->setText(::QObject::tr("Install NVIDIA closed source driver"));
     repaintDevices();
   } else {
     QFrame::changeEvent(event);
@@ -258,8 +246,6 @@ void AdvancedPartitionFrame::initConnections() {
           this, &AdvancedPartitionFrame::requestSelectBootloaderFrame);
   connect(editing_button_, &QPushButton::toggled,
           this, &AdvancedPartitionFrame::onEditButtonToggled);
-  connect(m_installNvidiaCheck, &QCheckBox::clicked,
-          this, &AdvancedPartitionFrame::installNvidiaStateChanged);
 }
 
 void AdvancedPartitionFrame::initUI() {
@@ -368,8 +354,6 @@ void AdvancedPartitionFrame::initUI() {
   QVBoxLayout* main_layout = new QVBoxLayout();
   main_layout->setContentsMargins(0, 0, 0, 0);
   main_layout->setSpacing(0);
-  main_layout->addWidget(msg_container_frame_, 0, Qt::AlignHCenter);
-  main_layout->addSpacing(8);
   QHBoxLayout* upLayout = new QHBoxLayout();
   upLayout->addWidget(scroll_area_);
   main_layout->addLayout(upLayout);
@@ -379,14 +363,6 @@ void AdvancedPartitionFrame::initUI() {
   main_layout->addLayout(downLayout);
   main_layout->addSpacing(8);
 
-  m_installNvidiaCheck = new QCheckBox;
-  m_installNvidiaCheck->setObjectName("check_box");
-  m_installNvidiaCheck->setCheckable(true);
-  m_installNvidiaCheck->setChecked(false);
-  m_installNvidiaCheck->setText(::QObject::tr("Install NVIDIA closed source driver"));
-  main_layout->addWidget(m_installNvidiaCheck, 0 ,Qt::AlignHCenter);
-
-  m_installNvidiaCheck->setVisible(delegate_->scanNvidia());
   WriteEnableNvidiaDriver(false);
 
   this->setLayout(main_layout);
@@ -534,6 +510,11 @@ void AdvancedPartitionFrame::repaintDevices() {
         continue;
       }
       AdvancedPartitionButton* button = new AdvancedPartitionButton(partition);
+      QString text = GetPartitionLabel(partition);
+      if (partition->type != PartitionType::Unallocated) {
+          text.append(QString("(%1)").arg(GetPartitionName(partition->path)));
+      }
+      button->setToolTip(QString("%1  %2  %3").arg(text, partition->path, GetFsTypeName(partition->fs)));
       button->setEditable(editing_button_->isChecked());
       partition_layout_->addWidget(button);
       partition_button_group_->addButton(button);
@@ -566,6 +547,9 @@ void AdvancedPartitionFrame::showErrorMessage(ValidateState state) {
   AdvancedPartitionErrorLabel* error_label =
       new AdvancedPartitionErrorLabel();
   error_label->setObjectName(kLastErrMsgLabel);
+  error_label->setAlignment(Qt::AlignLeft);
+  error_label->setWordWrap(true);
+  error_label->adjustSize();
 
   error_labels_.append(error_label);
 
@@ -573,7 +557,7 @@ void AdvancedPartitionFrame::showErrorMessage(ValidateState state) {
   const QString text = this->validateStateToText(state);
   error_label->setText(text);
   error_label->show();
-  error_label->setFixedHeight(kErrorMsgLabelHeight);
+
   msg_layout_->addWidget(error_label);
 
   connect(error_label, &AdvancedPartitionErrorLabel::entered,
@@ -671,12 +655,12 @@ QString AdvancedPartitionFrame::validateStateToText(ValidateState state) {
                 "can only be %1 ").arg(fs_name);
     }
     case ValidateState::BootPartNumberInvalid: {
-      return ::QObject::tr("The partition of /boot directory should be "
-                "the first partition on hard disk");
+      return ::QObject::tr("The partition of %1 directory should be the first partition on the hard disk"
+                           ", and starts at the beginning of the disk").arg("/boot");
     }
     case ValidateState::EfiPartNumberinvalid: {
-      return ::QObject::tr("The partition of /boot/efi directory should be "
-                "the first partition on hard disk");
+      return ::QObject::tr("The partition of %1 directory should be the first partition on the hard disk"
+                           ", and starts at the beginning of the disk").arg("/boot/efi");
     }
     case ValidateState::BootTooSmall: {
       const int boot_recommended = GetSettingsInt(kPartitionDefaultBootSpace);
@@ -735,6 +719,7 @@ void AdvancedPartitionFrame::onDeletePartitionTriggered(
     const Partition::Ptr partition) {
   removeOsProberDataByPath(partition->path);
 
+  delegate_->unFormatPartition(partition);//删除分区的时候清除原来设置的格式化分区的记录
   delegate_->deletePartition(partition);
   delegate_->refreshVisual();
 }
