@@ -27,6 +27,7 @@
 #include <QStackedLayout>
 #include <DSysInfo>
 #include <DFrame>
+#include <QProcess>
 
 #include "base/file_util.h"
 #include "partman/device.h"
@@ -121,14 +122,26 @@ bool FullDiskFrame::isEncrypt() const
     return m_encryptCheck->isChecked();
 }
 
-bool FullDiskFrame::isInstallNvidia() const
+bool FullDiskFrame::isEnSaveData() const
 {
-    return m_installNvidiaCheck->isChecked();
+    return m_saveDataCheck->isChecked();
+}
+
+void FullDiskFrame::setSaveDataCheckEnabel(bool isenabel)
+{
+    m_saveDataCheck->setEnabled(isenabel);
 }
 
 bool FullDiskFrame::focusSwitch()
 {
-    if(m_encryptCheck->hasFocus()) {
+    if (m_saveDataCheck->hasFocus()) {
+        if (m_encryptCheck->isVisible() && m_encryptCheck->isEnabled()) {
+            m_encryptCheck->setFocus();
+            return false;
+        } else {
+            return true;
+        }
+    } else if (m_encryptCheck->hasFocus()) {
         return true;
     } else {
         if (m_disk_layout->currentWidget() == m_diskInstallationWidget) {
@@ -145,8 +158,8 @@ bool FullDiskFrame::focusSwitch()
             }
         } else if (m_disk_layout->currentWidget() == m_grid_wrapper) {
             if(m_grid_wrapper->hasFocus()) {
-                if (m_encryptCheck->isVisible()) {
-                    m_encryptCheck->setFocus();
+                if (m_saveDataCheck->isVisible()) {
+                    m_saveDataCheck->setFocus();
                 } else {
                     return true;
                 }
@@ -160,7 +173,16 @@ bool FullDiskFrame::focusSwitch()
 
 bool FullDiskFrame::doSpace()
 {
-    if(m_encryptCheck->hasFocus()) {
+    if (m_saveDataCheck->hasFocus()) {
+        if (m_saveDataCheck->checkState() == Qt::Checked) {
+            m_saveDataCheck->setCheckState(Qt::Unchecked);
+            m_encryptCheck->setEnabled(true);
+        } else {
+            m_saveDataCheck->setCheckState(Qt::Checked);
+            m_encryptCheck->setCheckState(Qt::Unchecked);
+            m_encryptCheck->setEnabled(false);
+        }
+    } else if(m_encryptCheck->hasFocus()) {
         if (m_encryptCheck->checkState() == Qt::Checked) {
             m_encryptCheck->setCheckState(Qt::Unchecked);
         } else {
@@ -198,7 +220,7 @@ void FullDiskFrame::changeEvent(QEvent* event) {
         for (auto it = m_trList.begin(); it != m_trList.end(); ++it) {
             it->first(qApp->translate("QObject", it->second.toUtf8()));
         }
-        m_installNvidiaCheck->setText(::QObject::tr("Install NVIDIA closed source driver"));
+        m_saveDataCheck->setText(::QObject::tr("Keep User Data"));
         m_encryptCheck->setText(::QObject::tr("Encrypt This Disk"));
         m_errorTip->setText(::QObject::tr("Please select a disk to start installation"));
 
@@ -234,16 +256,23 @@ void FullDiskFrame::initConnections() {
           (&QButtonGroup::buttonToggled),
           this, &FullDiskFrame::onPartitionButtonToggled);
   connect(m_encryptCheck, &QCheckBox::clicked, this, &FullDiskFrame::cryptoStateChanged);
-  connect(m_installNvidiaCheck, &QCheckBox::clicked, this, &FullDiskFrame::installNvidiaStateChanged);
   connect(m_delegate, &FullDiskDelegate::deviceRefreshed,
           m_diskInstallationWidget, &MultipleDiskInstallationWidget::onDeviceListChanged);
   connect(m_diskInstallationWidget, &MultipleDiskInstallationWidget::currentDeviceChanged,
           this, &FullDiskFrame::onCurrentDeviceChanged);
+  connect(m_saveDataCheck, &QCheckBox::clicked, this, &FullDiskFrame::saveDataStateChanged);
 }
 
 void FullDiskFrame::initUI() {
   m_button_group = new QButtonGroup(this);
 
+  m_saveDataCheck = new QCheckBox;
+  m_saveDataCheck->setObjectName("check_box");
+  m_saveDataCheck->setCheckable(true);
+  m_saveDataCheck->setChecked(false);
+  m_saveDataCheck->setEnabled(false);
+  addTransLate(m_trList, std::bind(&QCheckBox::setText, m_saveDataCheck, std::placeholders::_1), ::QObject::tr("Save User Data"));
+
   m_encryptCheck = new QCheckBox;
   m_encryptCheck->setObjectName("check_box");
   m_encryptCheck->setCheckable(true);
@@ -251,13 +280,6 @@ void FullDiskFrame::initUI() {
   //m_encryptCheck->setFocusPolicy(Qt::TabFocus);
   addTransLate(m_trList, std::bind(&QCheckBox::setText, m_encryptCheck, std::placeholders::_1), ::QObject::tr("Encrypt This Disk"));
 
-  m_installNvidiaCheck = new QCheckBox;
-  m_installNvidiaCheck->setObjectName("check_box");
-  m_installNvidiaCheck->setCheckable(true);
-  m_installNvidiaCheck->setChecked(false);
-  //m_installNvidiaCheck->setFocusPolicy(Qt::TabFocus);
-  m_installNvidiaCheck->setText(::QObject::tr("Install NVIDIA closed source driver"));
-
   m_errorTip = new QLabel;
   m_errorTip->setObjectName("msg_label");
   m_errorTip->hide();
@@ -283,6 +305,7 @@ void FullDiskFrame::initUI() {
   m_tip_label = new QLabel("");
   m_tip_label->setObjectName("tip_label");
   m_tip_label->setPalette(palette);
+  m_tip_label->setForegroundRole(QPalette::Text);
 
   QHBoxLayout* tip_layout = new QHBoxLayout;
   tip_layout->setContentsMargins(0, 0, 0, 0);
@@ -334,7 +357,7 @@ void FullDiskFrame::initUI() {
 
   m_diskPartitionWidget = new FullDiskPartitionWidget;
   m_diskPartitionWidget->setObjectName("m_diskPartitionWidget");
-  m_diskPartitionWidget->setMinimumHeight(85);
+  m_diskPartitionWidget->setMinimumHeight(88);
 
   QVBoxLayout* main_layout = new QVBoxLayout();
   main_layout->setContentsMargins(0, 0, 0, 0);
@@ -343,8 +366,9 @@ void FullDiskFrame::initUI() {
   main_layout->addWidget(m_diskPartitionWidget, 0, Qt::AlignHCenter);
   main_layout->addStretch();
   QHBoxLayout* h_layout = new QHBoxLayout();
+  h_layout->addWidget(m_saveDataCheck);
   h_layout->addWidget(m_encryptCheck);
-  h_layout->addWidget(m_installNvidiaCheck);
+  h_layout->setSpacing(91);
   main_layout->addSpacing(10);
   main_layout->addLayout(h_layout);
   main_layout->addSpacing(10);
@@ -354,7 +378,6 @@ void FullDiskFrame::initUI() {
   main_layout->addSpacing(10);
 
   m_encryptCheck->setVisible(!GetSettingsBool(KPartitionSkipFullCryptPage));
-  m_installNvidiaCheck->setVisible(m_delegate->scanNvidia());
   WriteEnableNvidiaDriver(false);
 
   this->setLayout(main_layout);
@@ -423,6 +446,97 @@ void FullDiskFrame::showInstallTip(bool isshow) {
     }
 }
 
+bool FullDiskFrame::isExistDataPart(const QString &devicepath)
+{    
+    QString cmd = QString("lsblk -f %1 | grep _dde_data").arg(devicepath);
+    QProcess testprocess;
+    testprocess.start("/bin/bash",{"-c", cmd});
+    testprocess.waitForFinished();
+    QString dataindevices = testprocess.readAll();
+    qWarning() << "dataindevices = " << dataindevices;
+    if (dataindevices.compare("")) {
+        return true;
+    } else {
+        return false;
+    }
+}
+
+bool FullDiskFrame::isSystemDisk(const QString &devicepath)
+{
+    QString cmd = QString("lsblk -o LABEL -lnf %1 | grep [a-z/A-Z] | xargs").arg(devicepath);
+    QProcess testprocess;
+    testprocess.start("/bin/bash",{"-c", cmd});
+    testprocess.waitForFinished();
+    QString devicelabels = testprocess.readAll();
+    qWarning() << "devicelabels = " << devicelabels;
+    QStringList devicelabelslist = devicelabels.split(" ");
+
+    int sysytemdisknums = 0;
+
+    if (devicelabelslist.contains("Roota"))
+        sysytemdisknums++;
+
+    if (sysytemdisknums == 1) {
+        return true;
+    } else {
+        return false;
+    }
+}
+
+bool FullDiskFrame::isFullDiskEncrypt(const QString &devicepath)
+{
+    QString cmd = QString("lsblk -f %1 | grep crypto_LUKS").arg(devicepath);
+    QProcess testprocess;
+    testprocess.start("/bin/bash",{"-c", cmd});
+    testprocess.waitForFinished();
+    QString cryptodevices = testprocess.readAll();
+    qWarning() << "cryptodevices = " << cryptodevices;
+    if (cryptodevices.compare("")) {
+        return true;
+    } else {
+        return false;
+    }
+}
+
+void FullDiskFrame::setSaveDataCheckboxStat(const Device::Ptr device,  const int type)
+{
+    // 判断是否是一个系统盘
+    bool testissystemdisk = false;
+    if (static_cast<int>(DiskModelType::SystemDisk) == type) {
+        testissystemdisk = isSystemDisk(device->path);
+    } else if (static_cast<int>(DiskModelType::DataDisk) == type) {
+        testissystemdisk = isSystemDisk(m_delegate->selectedDisks().at(0));
+    }
+
+    bool testexistdata = isExistDataPart(device->path);
+    bool testfulldiskencrypt = isFullDiskEncrypt(device->path);
+
+    bool testsametabletype = true;
+    PartitionTableType currenttable = IsEfiEnabled() ? PartitionTableType::GPT : PartitionTableType::MsDos;
+    //如果直接使用传进来的device来判断table，存在table被修改的可能，所以这里从realdevice中找到需要的device
+    const DeviceList realDevices = m_delegate->realDevices();
+    for (int i = 0; i < realDevices.size(); i++) {
+        if (!device->path.compare(realDevices.at(i)->path)) {
+            // 检测磁盘格式和引导模式是否一致
+            testsametabletype = (realDevices.at(i)->table == currenttable) ? true : false;
+            qWarning() << "device table : " << realDevices.at(i)->table << " currenttable : " << currenttable;
+            break;
+        }
+    }
+
+    // 存在data分区并且在选择系统盘时所选盘存在系统分区
+    // 或者存在data分区并且选择数据盘时，选择的系统盘存在系统分区
+    if ( testexistdata && (!testfulldiskencrypt) && testsametabletype && testissystemdisk ) {
+        m_saveDataCheck->setEnabled(!m_encryptCheck->isChecked());
+        emit showSaveDataPopWidget();
+    } else {
+        m_saveDataCheck->setChecked(false);
+        m_saveDataCheck->setEnabled(false);
+        m_encryptCheck->setEnabled(true);
+        WriteSaveUserData(false);
+    }
+}
+
 void FullDiskFrame::onDeviceRefreshed() {
   this->repaintDevices();
   m_delegate->removeAllSelectedDisks();
@@ -451,6 +565,10 @@ void FullDiskFrame::onPartitionButtonToggled(QAbstractButton* button,
   } else {    
 
     m_delegate->addSystemDisk(part_button->device()->path);
+
+    // 设置保留用户数据勾选控件
+    setSaveDataCheckboxStat(part_button->device());
+
     const QString path = part_button->device()->path;
     qDebug() << "selected device path:" << path;
     part_button->setSelected(true);
@@ -478,6 +596,9 @@ void FullDiskFrame::onCurrentDeviceChanged(int type, const Device::Ptr device)
         qWarning() << QString("MULTIDISK:invalid type:{%1}").arg(type);
     }
 
+    // 设置保留用户数据勾选控件
+    setSaveDataCheckboxStat(device, type);
+
     // If validate device failed, then clear last device partition info in the widget.
     if(!validate()) {
         qWarning() << QString("MULTIDISK:validate FAILED:{%1}").arg(m_delegate->selectedDisks().join(","));
@@ -489,9 +610,35 @@ void FullDiskFrame::onCurrentDeviceChanged(int type, const Device::Ptr device)
     }
 }
 
-void FullDiskFrame::installNvidiaStateChanged(bool install_nvidia)
+void FullDiskFrame::saveDataStateChanged(bool savedata)
 {
-    WriteEnableNvidiaDriver(install_nvidia);
+    if (savedata) {
+        m_saveDataCheck->setEnabled(true);
+        m_encryptCheck->setChecked(false);
+        m_encryptCheck->setEnabled(false);
+    } else {
+        m_encryptCheck->setEnabled(true);
+    }
+
+    m_saveDataCheck->setChecked(savedata);
+    WriteSaveUserData(savedata);
+}
+
+void FullDiskFrame::cryptoStateChanged(bool crypto)
+{
+    if (crypto) {
+        m_saveDataCheck->setChecked(false);
+        m_saveDataCheck->setEnabled(false);
+        WriteSaveUserData(false);
+    } else {
+        // 当撤销“全盘加密”的勾选状态时，“保留用户数据”的可用状态根据重新判断所选的系统盘和数据盘确定
+        DeviceList testdevicelist = m_delegate->selectedDevices();
+        for (int i = 0; i < testdevicelist.size(); i++) {
+            setSaveDataCheckboxStat(testdevicelist.at(i), i);
+        }
+    }
+
+    m_encryptCheck->setChecked(crypto);
 }
 
 }  // namespace installer
