@@ -101,6 +101,14 @@ private:
     // Read xkb layout list with |locale|.
     void initLayout(const QString& locale);
 
+    // Clear user last checked keyboard layout and variant model index,
+    // because keyboard layout and variant must be keep up with language
+    // when user checked different language.
+    void resetLastCheckedModelIndex();
+
+    // Update keyboard layout and variant by read locale or setting file.
+    void updateViewCheckedByLocale();
+
     // Get variant list at |index|.
     XKbLayoutVariantList getVariantList(const QModelIndex& index);
 
@@ -129,21 +137,15 @@ private:
     DListView* m_layoutView = new TipListView;
     DListView* m_variantView = new TipListView;
 
-    // The user last checked item.
-    DStandardItem* m_lastItem = nullptr;
-    DStandardItem* m_lastItemVar = nullptr;
-
+    // The user last checked layout item.
     QModelIndex m_lastMode;
+    // The user last checked layout variant item.
     QModelIndex m_lastModeVar;
 
     // Keyboard layout list sorted by description.
     XkbLayoutList layout_list_;
     XKbLayoutVariantList variant_list_;
     XkbModelList model_list;
-
-    // The user last checked both left and right view item.
-    DStandardItem* m_lastChangedItem = nullptr;
-    int m_lastItemRow = -1;
 };
 
 SystemInfoKeyboardFrame::SystemInfoKeyboardFrame(FrameProxyInterface *parent)
@@ -298,10 +300,6 @@ void SystemInfoKeyboardFramePrivate::initLayout(const QString& locale) {
   layout_list_ = xkb_config_.layout_list;
 
   m_layoutModel->clear();
-  m_lastItem = nullptr;
-  m_lastChangedItem = nullptr;
-  m_lastItemVar = nullptr;
-  m_lastItemRow = -1;
 
   // Sort layout list by description.
   // Perform localized comparison.
@@ -321,6 +319,32 @@ void SystemInfoKeyboardFramePrivate::initLayout(const QString& locale) {
   m_layoutView->update();
 }
 
+void SystemInfoKeyboardFramePrivate::resetLastCheckedModelIndex()
+{
+    m_lastMode = QModelIndex();
+    m_lastModeVar = QModelIndex();
+}
+
+void SystemInfoKeyboardFramePrivate::updateViewCheckedByLocale()
+{
+    QStringList localeList;
+    localeList << ReadLocale() << GetSettingsString(kSelectLanguageDefaultLocale);
+
+    for (const QString& locale : localeList) {
+        int index = locale.indexOf('_');
+        if (index >= 0){
+            const QModelIndex modelIndex = getLayoutByName(locale.mid(index + 1).toLower());
+            if (modelIndex.isValid()) {
+                m_layoutView->setCurrentIndex(modelIndex);
+                break;
+            }
+        }
+        else{
+            qWarning() << "invalid locale:" << locale;
+        }
+    }
+}
+
 XKbLayoutVariantList SystemInfoKeyboardFramePrivate::getVariantList(
     const QModelIndex& index) {
   if (index.isValid()) {
@@ -463,6 +487,14 @@ void SystemInfoKeyboardFrame::changeEvent(QEvent* event) {
     if (event->type() == QEvent::LanguageChange) {
         d->updateTs();
         d->initLayout(ReadLocale());
+
+        // Clear user last checked keyboard layout and variant model index
+        // when user change language, and then update keyboard layout view
+        // and variant view by read locale or setting file.
+        d->resetLastCheckedModelIndex();
+        d->updateViewCheckedByLocale();
+
+        finished();
     } else {
         FrameInterface::changeEvent(event);
     }
@@ -471,21 +503,25 @@ void SystemInfoKeyboardFrame::changeEvent(QEvent* event) {
 void SystemInfoKeyboardFrame::showEvent(QShowEvent *event)
 {
     Q_D(SystemInfoKeyboardFrame);
-    QStringList localeList;
-    localeList << ReadLocale() << GetSettingsString(kSelectLanguageDefaultLocale);
 
-    for (const QString& locale : localeList) {
-        int index = locale.indexOf('_');
-        if (index >= 0){
-            const QModelIndex modelIndex = d->getLayoutByName(locale.mid(index + 1).toLower());
-            if (modelIndex.isValid()) {
-                d->m_layoutView->setCurrentIndex(modelIndex);
-                break;
-            }
-        }
-        else{
-            qWarning() << "invalid locale:" << locale;
-        }
+    DStandardItem* tmpItem = nullptr;
+    DStandardItem* tmpVarItem = nullptr;
+
+    if (d->m_lastMode.isValid() && d->m_lastModeVar.isValid()) {
+        tmpItem = dynamic_cast<DStandardItem* >(d->m_layoutModel->item(d->m_lastMode.row()));
+        tmpVarItem = dynamic_cast<DStandardItem* >(d->m_variantModel->item(d->m_lastModeVar.row()));
+    }
+
+    if (tmpItem != nullptr && tmpVarItem != nullptr) {
+        qInfo() << "Set last checked layout and variant when page show";
+
+        tmpItem->setCheckState(Qt::Checked);
+        tmpVarItem->setCheckState(Qt::Checked);
+    }
+    else {
+        qInfo() << "Set layout and variant from locale or setting file when page show";
+
+        d->updateViewCheckedByLocale();
     }
 
     this->setCurentFocus(d_private->nextButton);
@@ -726,9 +762,6 @@ void SystemInfoKeyboardFramePrivate::onVariantViewSelected(
         item->setCheckState(Qt::Checked);
     }
 
-    m_lastChangedItem = dynamic_cast<DStandardItem* >
-            (m_layoutModel->item(m_layoutView->currentIndex().row()));
-
     const QModelIndex layout_index = m_layoutView->currentIndex();
     const QString variant = getVariantName(current);
     const QString layout = getLayoutName(current);
