@@ -24,7 +24,6 @@
 #include "ui/delegates/full_disk_delegate.h"
 #include "ui/delegates/simple_partition_delegate.h"
 #include "ui/frames/dynamic_disk_warning_frame.h"
-#include "ui/frames/swap_warnning_frame.h"
 #include "ui/delegates/partition_util.h"
 #include "ui/frames/consts.h"
 #include "ui/frames/inner/advanced_partition_frame.h"
@@ -41,6 +40,7 @@
 #include "ui/frames/inner/select_bootloader_frame.h"
 #include "ui/frames/inner/simple_partition_frame.h"
 #include "ui/frames/inner/full_disk_encrypt_frame.h"
+#include "ui/frames/warnning_frame.h"
 #include "ui/models/partition_model.h"
 #include "ui/widgets/comment_label.h"
 #include "ui/widgets/title_label.h"
@@ -98,7 +98,6 @@ public:
 
      void onButtonGroupToggled(QAbstractButton *button);
      void onNextButtonClicked();
-     void onFullDiskCryptoButtonClicked(bool encrypto);
 
      // Write partitioning settings to conf file and emit manualPartDone() signal.
      void onManualPartDone(bool ok, const DeviceList& devices);
@@ -117,8 +116,11 @@ public:
      void showEncryptFrame();
      void showDynamicDiskFrame();
      void showPrepareInstallFrame();
+     void ShowSaveDataPopWidget();
 
      bool isEncrypt();
+     bool isEnSaveData();
+     bool isLvm();
 
      PartitionFrame* q_ptr=nullptr;
 
@@ -139,6 +141,8 @@ public:
      SimplePartitionFrame* simple_partition_frame_ = nullptr;
      Full_Disk_Encrypt_frame* full_disk_encrypt_frame_ = nullptr;
      DynamicDiskWarningFrame* dynamic_disk_warning_frame_ = nullptr;
+     WarnningFrame*   save_data_pop_widget = nullptr;
+     WarnningFrame*   swap_warnning_frame = nullptr;
 
      TitleLabel* title_label_ = nullptr;
      CommentLabel* comment_label_ = nullptr;
@@ -202,13 +206,19 @@ void PartitionFrame::autoPart() {
     if (GetSettingsBool("DI_LUPIN")) {
         emit finished();
         emit autoPartDone(true);
-        return;
     }
-
-    if (m_private->isEncrypt()) {
+    else if (m_private->isEncrypt()) {
         WriteFullDiskMode(true);
         m_private->partition_model_->autoPart();
     }
+    else if (m_private->isLvm()) {
+        m_private->setupDiskEncrypt(false);
+        m_private->partition_model_->autoPart();
+    }
+    else if (m_private->isEnSaveData()) {
+        m_private->setupDiskEncrypt(false);
+        m_private->partition_model_->autoPart();
+    }
     else {
         m_private->full_disk_delegate_->setAutoInstall(true);
         scanDevices();
@@ -230,7 +240,22 @@ void PartitionFrame::onAutoInstallPrepareFinished(bool finished)
 
     OperationList list = m_private->full_disk_delegate_->operations();
 
-    m_private->partition_model_->manualPart(list);
+    if (GetSettingsBool(kPartitionDoAutoPartCrypt)) {
+        m_private->setupDiskEncrypt(true);
+        WriteFullDiskEncryptPassword(GetSettingsString(kDiskCryptPassword));
+        m_private->partition_model_->autoPart();
+    }
+    else if (m_private->isLvm()) {
+        m_private->setupDiskEncrypt(false);
+        m_private->partition_model_->autoPart();
+    }
+    else if (GetSettingsBool(kMultiSystemEanble)) {
+        m_private->setupDiskEncrypt(false);
+        m_private->partition_model_->autoPart();
+    }
+    else {
+        m_private->partition_model_->manualPart(list);
+    }
 }
 
 void PartitionFrame::changeEvent(QEvent* event) {
@@ -247,6 +272,11 @@ void PartitionFrame::changeEvent(QEvent* event) {
           m_private->nextButton->setText(::QObject::tr("Next"));
       }
 
+      m_private->swap_warnning_frame->setTitle(::QObject::tr("Friendly Note"));
+      m_private->swap_warnning_frame->setComment(::QObject::tr("No swap partition created, which may affect system performance"));
+      m_private->swap_warnning_frame->setEnterButtonText(::QObject::tr("OK"));
+      m_private->swap_warnning_frame->setCancelButtonText(::QObject::tr("Cancel"));
+
     m_private->simple_frame_button_->setText(::QObject::tr("Simple"));
     m_private->advanced_frame_button_->setText(::QObject::tr("Advanced"));
     m_private->full_disk_frame_button_->setText(::QObject::tr("Full Disk"));
@@ -520,15 +550,19 @@ void PartitionFramePrivate::initConnections() {
 
   connect(edit_partition_frame_, &EditPartitionFrame::finished, this, [=] {
       q_ptr->m_proxy->hideChildFrame();
+      q_ptr->repaint();
   });
   connect(edit_lvm_partition_frame_, &EditPartitionFrame::finished, this, [=] {
       q_ptr->m_proxy->hideChildFrame();
+      q_ptr->repaint();
   });
   connect(new_partition_frame_, &NewPartitionFrame::finished, this, [=] {
       q_ptr->m_proxy->hideChildFrame();
+      q_ptr->repaint();
   });
   connect(new_lvm_partition_frame_, &NewPartitionFrame::finished, this, [=] {
       q_ptr->m_proxy->hideChildFrame();
+      q_ptr->repaint();
   });
 
   connect(new_table_warning_frame_, &NewTableWarningFrame::canceled,
@@ -553,11 +587,19 @@ void PartitionFramePrivate::initConnections() {
   connect(prepare_install_frame_, &PrepareInstallFrame::aborted,
           this, &PartitionFramePrivate::showMainFrame);
   connect(prepare_install_frame_, &PrepareInstallFrame::finished, this, [=] {
-      if (!GetSettingsBool(KPartitionSkipFullCryptPage) && this->isEncrypt()) {
-        q_ptr->autoPart();
-        q_ptr->m_proxy->nextFrame();
-      }
-      else {
+      // TODO: this->isEncrypt() == true ==> !GetSettingsBool(KPartitionSkipFullCryptPage) == true
+      // so can delete !GetSettingsBool(KPartitionSkipFullCryptPage).
+      // Another place to be modified is function PartitionFramePrivate::showEncryptFrame().
+      if ((!GetSettingsBool(KPartitionSkipFullCryptPage) && this->isEncrypt())
+              || this->isEnSaveData()
+              || this->isLvm()) {
+          q_ptr->autoPart();
+          q_ptr->m_proxy->nextFrame();
+      } else if (GetSettingsBool(kMultiSystemEanble)) {
+          setupDiskEncrypt(false);
+          partition_model_->autoPart();
+          q_ptr->m_proxy->nextFrame();
+      } else {
         onPrepareInstallFrameFinished();
       }
   });
@@ -569,6 +611,7 @@ void PartitionFramePrivate::initConnections() {
           advanced_delegate_, &AdvancedPartitionDelegate::setBootloaderPath);
   connect(select_bootloader_frame_, &SelectBootloaderFrame::finished, this, [=] {
       q_ptr->m_proxy->hideChildFrame();
+      q_ptr->repaint();
   });
   connect(advanced_delegate_, &AdvancedPartitionDelegate::deviceRefreshed,
           select_bootloader_frame_, &SelectBootloaderFrame::deviceRefreshed);
@@ -605,6 +648,7 @@ void PartitionFramePrivate::initConnections() {
 
   connect(full_disk_encrypt_frame_, &Full_Disk_Encrypt_frame::cancel, this, [=] {
       q_ptr->m_proxy->hideChildFrame();
+      q_ptr->repaint();
   });
 
   connect(full_disk_encrypt_frame_, &Full_Disk_Encrypt_frame::encryptFinished, q_ptr, [=] {
@@ -613,6 +657,7 @@ void PartitionFramePrivate::initConnections() {
       setupDiskEncrypt(this->isEncrypt()); // 设置全盘分区场景下的全盘加密
 
       q_ptr->m_proxy->hideChildFrame();
+      q_ptr->repaint();
       showPrepareInstallFrame();
   });
 
@@ -625,15 +670,24 @@ void PartitionFramePrivate::initConnections() {
   connect(full_disk_partition_frame_, &FullDiskFrame::showDeviceInfomation,
           full_disk_encrypt_frame_, &Full_Disk_Encrypt_frame::onShowDeviceInfomation);
 
-  connect(full_disk_partition_frame_, &FullDiskFrame::cryptoStateChanged,
-          this, &PartitionFramePrivate::onFullDiskCryptoButtonClicked);
-
   connect(full_disk_partition_frame_, &FullDiskFrame::enableNextButton, [=](const bool& enable) {
       nextButton->setEnabled(enable);
   });
 
+  connect(full_disk_partition_frame_, &FullDiskFrame::showSaveDataPopWidget, this, &PartitionFramePrivate::ShowSaveDataPopWidget);
+
   connect(full_disk_delegate_, &FullDiskDelegate::requestAutoInstallFinished, q_ptr, &PartitionFrame::onAutoInstallPrepareFinished);
 
+  connect(swap_warnning_frame, &WarnningFrame::quitEntered, this, [=] {
+      q_ptr->m_proxy->hideChildFrame();
+      q_ptr->repaint();
+  });
+
+  connect(swap_warnning_frame, &WarnningFrame::quitCanceled, this, [=] {
+      q_ptr->m_proxy->hideChildFrame();
+      q_ptr->repaint();
+  });
+
     Q_EMIT full_disk_frame_button_->click();
 }
 
@@ -660,6 +714,10 @@ void PartitionFramePrivate::initUI() {
 
   dynamic_disk_warning_frame_ = new DynamicDiskWarningFrame(q_ptr);
 
+  swap_warnning_frame = new WarnningFrame(q_ptr->m_proxy);
+  swap_warnning_frame->useCancelButton(false);
+  swap_warnning_frame->hide();
+
   title_label_ = new TitleLabel(::QObject::tr("Create Partitions"));
   title_label_->setObjectName("title_label_");
   comment_label_ = new CommentLabel(
@@ -887,7 +945,7 @@ void PartitionFramePrivate::onNextButtonClicked() {
     if (!advanced_partition_frame_->validate()) {
       return;
     }
-  } else if (AdvancedPartitionDelegate::install_Lvm_Status == Install_Lvm_Status::Lvm_Install) {
+  } else {
     // Validate advanced partition frame.
     if (!lvm_partition_frame_->validate()) {
       return;
@@ -905,8 +963,7 @@ void PartitionFramePrivate::onNextButtonClicked() {
     dynamic_disk_warning_frame_->setWarningTip(::QObject::tr("The target disk is dynamic which will be formatted if proceeding. Please make a backup of your important files first."));
     static bool isFirstWarning = true;
     if (isFirstWarning && !AdvancedPartitionDelegate::swapOk && AdvancedPartitionDelegate::install_Lvm_Status != Install_Lvm_Status::Lvm_Format_Pv) {
-        SwapWarnningFrame swapWarnningFrame;
-        swapWarnningFrame.display();
+        q_ptr->m_proxy->showChildFrame(swap_warnning_frame);
         isFirstWarning = false;
         qApp->setActiveWindow(q_ptr);
         return;
@@ -921,22 +978,16 @@ void PartitionFramePrivate::onNextButtonClicked() {
     return;
   }
 
-    showPrepareInstallFrame();
-
-    if (AdvancedPartitionDelegate::install_Lvm_Status == Install_Lvm_Status::Lvm_Format_Pv) {
-       emit prepare_install_frame_->finished();
-       lvm_partition_frame_->updateLayout(next_layout, ::QObject::tr("Back"));
+    if ((partition_stacked_layout_->currentWidget() == advanced_partition_frame_ )
+            && (AdvancedPartitionDelegate::install_Lvm_Status == Install_Lvm_Status::Lvm_Format_Pv)) {
+       lvm_partition_frame_->updateLayout(next_layout, ::QObject::tr("Back", "button"));
        nextButton->setText(::QObject::tr("Ready to Install"));
-       emit q_ptr->coverMainWindowFrameLabelsView(false);
+       emit prepare_install_frame_->finished();   // 该信号会触发显示lvm的界面
+    } else {
+       showPrepareInstallFrame();
     }
 }
 
-void PartitionFramePrivate::onFullDiskCryptoButtonClicked(bool encrypto)
-{
-//    nextButton->setText(encrypto ? ::QObject::tr("Next") :
-//                                   ::QObject::tr("Start installation"));
-}
-
 void PartitionFramePrivate::onManualPartDone(bool ok, const DeviceList& devices) {
   if (ok) {
 
@@ -974,7 +1025,7 @@ void PartitionFramePrivate::onPrepareInstallFrameFinished() {
         found_boot = full_disk_delegate_->setBootFlag();
     } else if (AdvancedPartitionDelegate::install_Lvm_Status != Install_Lvm_Status::Lvm_Install) {
         found_boot = advanced_delegate_->setBootFlag();
-    } else if (AdvancedPartitionDelegate::install_Lvm_Status == Install_Lvm_Status::Lvm_Install) {
+    } else {
         found_boot = lvm_delegate_->setBootFlag();
     }
 
@@ -991,14 +1042,14 @@ void PartitionFramePrivate::onPrepareInstallFrameFinished() {
         operations = full_disk_delegate_->operations();
     } else if (AdvancedPartitionDelegate::install_Lvm_Status != Install_Lvm_Status::Lvm_Install){
         operations = advanced_delegate_->operations();
-    } else if (AdvancedPartitionDelegate::install_Lvm_Status == Install_Lvm_Status::Lvm_Install){
+    } else {
         //注意一定要先添加老分区操作，否则会产生不可预料的后果
         operations = lvm_delegate_->m_oldOperationList;
         operations.append(lvm_delegate_->operations());
     }
 
     // full disk encrypt operations is empty.
-    if (isFullDiskPartitionMode() && this->isEncrypt()) {
+    if (isEncrypt()) {
         q_ptr->m_proxy->nextFrame();
     }
     else if (operations.isEmpty()) {
@@ -1007,7 +1058,7 @@ void PartitionFramePrivate::onPrepareInstallFrameFinished() {
     }
     else {
         partition_model_->manualPart(operations);
-        if (AdvancedPartitionDelegate::install_Lvm_Status != Install_Lvm_Status::Lvm_Format_Pv) {
+        if (isFullDiskPartitionMode() || AdvancedPartitionDelegate::install_Lvm_Status != Install_Lvm_Status::Lvm_Format_Pv) {
             q_ptr->m_proxy->nextFrame();
         } else {
             lvm_delegate_->m_oldOperationList = operations;
@@ -1033,7 +1084,7 @@ void PartitionFramePrivate::showMainFrame() {
       advanced_delegate_->onDeviceRefreshed(advanced_delegate_->realDevices());
       partition_stacked_layout_->setCurrentWidget(advanced_partition_frame_);
       main_layout_->setCurrentWidget(main_frame_);
-      title_label_->setText(::QObject::tr("Create Partitions"));     
+      title_label_->setText(::QObject::tr("Create Partitions"));
       nextButton->setText(::QObject::tr("Next"));
       comment_label_->setText(
           ::QObject::tr("Make sure you have backed up important data, then select the disk to install"));
@@ -1071,7 +1122,7 @@ void PartitionFramePrivate::showNewTableWarningFrame(const QString& device_path)
     return;
   } else if (AdvancedPartitionDelegate::install_Lvm_Status != Install_Lvm_Status::Lvm_Install) {
     devices = advanced_delegate_->realDevices();
-  } else if (AdvancedPartitionDelegate::install_Lvm_Status == Install_Lvm_Status::Lvm_Install) {
+  } else {
     devices = lvm_delegate_->realDevices();
   }
 
@@ -1115,6 +1166,7 @@ void PartitionFramePrivate::showEncryptFrame()
             q_ptr->m_proxy->showChildFrame(full_disk_encrypt_frame_);
         }
         else {
+            // TODO: code can nerver get here.
             q_ptr->autoPart();
             onPrepareInstallFrameFinished();
         }
@@ -1137,7 +1189,7 @@ void PartitionFramePrivate::showPrepareInstallFrame()
         descriptions = full_disk_delegate_->getOptDescriptions();
     } else if (AdvancedPartitionDelegate::install_Lvm_Status != Install_Lvm_Status::Lvm_Install) {
       descriptions = advanced_delegate_->getOptDescriptions();
-    } else if (AdvancedPartitionDelegate::install_Lvm_Status == Install_Lvm_Status::Lvm_Install) {
+    } else {
       descriptions = lvm_delegate_->getOptDescriptions();
     }
 
@@ -1149,10 +1201,44 @@ void PartitionFramePrivate::showPrepareInstallFrame()
     emit q_ptr->coverMainWindowFrameLabelsView(true);
 }
 
+void PartitionFramePrivate::ShowSaveDataPopWidget()
+{
+    if ( save_data_pop_widget == nullptr ) {
+        save_data_pop_widget = new WarnningFrame(nullptr);
+        save_data_pop_widget->useTitle(false);
+        // 因为是后创建的对象所以这个地方的设置文案操作不需要放在changeEvent里去
+        save_data_pop_widget->setComment(::QObject::tr("The \"/data/home\" directory is found. If you do not keep it, the data saved in it by previous users will be lost. Keep or delete it?"));
+        save_data_pop_widget->setEnterButtonText(::QObject::tr("Keep"));
+        save_data_pop_widget->setCancelButtonText(::QObject::tr("Delete"));
+        save_data_pop_widget->hide();
+        connect(save_data_pop_widget, &WarnningFrame::quitEntered, this, [=] {
+            full_disk_partition_frame_->saveDataStateChanged(true);
+            q_ptr->m_proxy->hideChildFrame();
+            q_ptr->repaint();
+        });
+        connect(save_data_pop_widget, &WarnningFrame::quitCanceled,this, [=] {
+            full_disk_partition_frame_->saveDataStateChanged(false);
+            q_ptr->m_proxy->hideChildFrame();
+            q_ptr->repaint();
+        });
+        q_ptr->m_proxy->showChildFrame(save_data_pop_widget);
+    }
+}
+
 bool PartitionFramePrivate::isEncrypt()
 {
     return isFullDiskPartitionMode() && full_disk_partition_frame_->isEncrypt();
 }
 
+bool PartitionFramePrivate::isEnSaveData()
+{
+    return isFullDiskPartitionMode() && full_disk_partition_frame_->isEnSaveData();
+}
+
+bool PartitionFramePrivate::isLvm()
+{
+    return isFullDiskPartitionMode() && full_disk_delegate_->isLvm();
+}
+
 }  // namespace installer
 #include "partition_frame.moc"
